@@ -1,28 +1,28 @@
 from __future__ import annotations
 
+import functools
 import io
 import typing as t
-import functools
 from urllib.parse import quote
 
-from starlette.requests import Request
-from multipart.multipart import parse_options_header
-from starlette.responses import Response
+from python_multipart.multipart import parse_options_header
 from starlette.datastructures import UploadFile
+from starlette.requests import Request
+from starlette.responses import Response
 
-from .base import IODescriptor
-from ..types import LazyType
-from ..utils import LazyLoader
-from ..utils import resolve_user_filepath
-from ..utils.http import set_cookies
 from ...exceptions import BadInput
-from ...exceptions import InvalidArgument
 from ...exceptions import InternalServerError
+from ...exceptions import InvalidArgument
 from ...exceptions import MissingDependencyException
 from ...grpc.utils import import_generated_stubs
 from ..service.openapi import SUCCESS_DESCRIPTION
-from ..service.openapi.specification import Schema
 from ..service.openapi.specification import MediaType
+from ..service.openapi.specification import Schema
+from ..types import LazyType
+from ..utils.filesystem import resolve_user_filepath
+from ..utils.http import set_cookies
+from ..utils.lazy_loader import LazyLoader
+from .base import IODescriptor
 
 PIL_EXC_MSG = "'Pillow' is required to use the Image IO descriptor. Install with 'pip install bentoml[io-image]'."
 
@@ -32,11 +32,11 @@ if t.TYPE_CHECKING:
     import PIL
     import PIL.Image
 
-    from .. import external_typing as ext
-    from .base import OpenAPIResponse
-    from ..context import InferenceApiContext as Context
     from ...grpc.v1 import service_pb2 as pb
     from ...grpc.v1alpha1 import service_pb2 as pb_v1alpha1
+    from .. import external_typing as ext
+    from ..context import ServiceContext as Context
+    from .base import OpenAPIResponse
 
     _Mode = t.Literal[
         "1", "CMYK", "F", "HSV", "I", "L", "LAB", "P", "RGB", "RGBA", "RGBX", "YCbCr"
@@ -75,7 +75,9 @@ def initialize_pillow():
 
     PIL.Image.init()
     MIME_EXT_MAPPING = {v: k for k, v in PIL.Image.MIME.items()}  # type: ignore (lazy constant)
-    READABLE_MIMES = {k for k, v in MIME_EXT_MAPPING.items() if v not in PIL_WRITE_ONLY_FORMATS}  # type: ignore (lazy constant)
+    READABLE_MIMES = {
+        k for k, v in MIME_EXT_MAPPING.items() if v not in PIL_WRITE_ONLY_FORMATS
+    }  # type: ignore (lazy constant)
 
 
 class Image(
@@ -106,7 +108,7 @@ class Image(
 
        runner = bentoml.tensorflow.get('image-classification:latest').to_runner()
 
-       svc = bentoml.Service("vit-object-detection", runners=[runner])
+       svc = bentoml.legacy.Service("vit-object-detection", runners=[runner])
 
        @svc.api(input=Image(), output=NumpyNdarray(dtype="float32"))
        async def predict_image(f: Image) -> NDArray[Any]:
@@ -194,17 +196,6 @@ class Image(
                 f"Invalid Image mime_type '{mime_type}'; supported mime types are {', '.join(PIL.Image.MIME.values())} "
             )
 
-        for mtype in self._allowed_mimes:
-            if mtype not in MIME_EXT_MAPPING:  # pragma: no cover
-                raise InvalidArgument(
-                    f"Invalid Image MIME in allowed_mime_types: '{mtype}'; supported mime types are {', '.join(PIL.Image.MIME.values())} "
-                )
-
-            if mtype not in READABLE_MIMES:
-                raise InvalidArgument(
-                    f"Pillow does not support reading '{mtype}' files."
-                )
-
         self._pilmode: _Mode | None = pilmode
         self._format: str = MIME_EXT_MAPPING[self._mime_type]
 
@@ -437,7 +428,7 @@ class Image(
                         )
                 except KeyError:
                     raise BadInput(
-                        f"{field.kind} is not a valid File kind. Accepted file kind: {[names for names,_ in pb_v1alpha1.File.FileType.items()]}",
+                        f"{field.kind} is not a valid File kind. Accepted file kind: {[names for names, _ in pb_v1alpha1.File.FileType.items()]}",
                     ) from None
             if not field.content:
                 raise BadInput("Content is empty!") from None
