@@ -1,16 +1,16 @@
 from __future__ import annotations
 
+import inspect
 import re
 import typing as t
-import inspect
-from typing import Optional
 
 import yaml
 
-from ..types import is_compatible_type
-from ..context import InferenceApiContext as Context
 from ...exceptions import InvalidArgument
+from ..context import ServiceContext as Context
 from ..io_descriptors import IODescriptor
+from ..io_descriptors.base import IOType
+from ..types import is_compatible_type
 
 RESERVED_API_NAMES = [
     "index",
@@ -23,15 +23,15 @@ RESERVED_API_NAMES = [
 ]
 
 
-class InferenceAPI:
+class InferenceAPI(t.Generic[IOType]):
     def __init__(
         self,
         user_defined_callback: t.Callable[..., t.Any] | None,
-        input_descriptor: IODescriptor[t.Any],
-        output_descriptor: IODescriptor[t.Any],
-        name: Optional[str],
-        doc: Optional[str] = None,
-        route: Optional[str] = None,
+        input_descriptor: IODescriptor[IOType],
+        output_descriptor: IODescriptor[IOType],
+        name: str | None,
+        doc: str | None = None,
+        route: str | None = None,
     ):
         if user_defined_callback is not None:
             # Use user_defined_callback function variable if name not specified
@@ -39,7 +39,8 @@ class InferenceAPI:
             # Use user_defined_callback function docstring `__doc__` if doc not specified
             doc = user_defined_callback.__doc__ if doc is None else doc
         else:
-            name, doc = "", ""
+            name = "" if name is None else name
+            doc = "" if doc is None else doc
 
         # Use API name as route if route not specified
         route = name if route is None else route
@@ -50,7 +51,7 @@ class InferenceAPI:
 
         if user_defined_callback is not None:
             InferenceAPI._validate_name(name)
-            InferenceAPI._validate_route(route)
+            InferenceAPI.validate_route(route)
 
             sig = inspect.signature(user_defined_callback)
 
@@ -103,11 +104,11 @@ class InferenceAPI:
                 if (
                     isinstance(annotation, t.Type)
                     and annotation != inspect.Signature.empty
+                    and not is_compatible_type(input_type, annotation)
                 ):
-                    if not is_compatible_type(input_type, annotation):
-                        raise TypeError(
-                            f"Expected type of argument '{first_arg}' to be '{input_type}', got '{sig.parameters[first_arg].annotation}'"
-                        )
+                    raise TypeError(
+                        f"Expected type of argument '{first_arg}' to be '{input_type}', got '{sig.parameters[first_arg].annotation}'"
+                    )
 
                 if len(sig.parameters) > 2:
                     raise ValueError(
@@ -121,11 +122,11 @@ class InferenceAPI:
                     if (
                         isinstance(annotation, t.Type)
                         and annotation != inspect.Signature.empty
+                        and not annotation == Context
                     ):
-                        if not annotation == Context:
-                            raise TypeError(
-                                f"Expected type of argument '{second_arg}' to be '{input_type}', got '{sig.parameters[second_arg].annotation}'"
-                            )
+                        raise TypeError(
+                            f"Expected type of argument '{second_arg}' to be 'bentoml.Context', got '{annotation}'"
+                        )
 
         if user_defined_callback is not None:
             self.func = user_defined_callback
@@ -162,7 +163,7 @@ class InferenceAPI:
             )
 
     @staticmethod
-    def _validate_route(route: str):
+    def validate_route(route: str):
         if re.findall(
             r"[?#]+|^(//)|^:", route
         ):  # contains '?' or '#' OR  start with '//' OR start with ':'
@@ -176,7 +177,7 @@ class InferenceAPI:
             )
 
 
-def _InferenceAPI_dumper(dumper: yaml.Dumper, api: InferenceAPI) -> yaml.Node:
+def _InferenceAPI_dumper(dumper: yaml.Dumper, api: InferenceAPI[t.Any]) -> yaml.Node:
     return dumper.represent_dict(
         {
             "route": api.route,
