@@ -35,25 +35,27 @@ class Arbiter(_Arbiter):
         return super().stop()
 
 
-class ThreadedArbiter(Arbiter, Thread):
+class ThreadedArbiter(Arbiter):
     def __init__(self, *args: t.Any, **kwargs: t.Any) -> None:
-        Arbiter.__init__(self, *args, **kwargs)
-        Thread.__init__(self, daemon=True)
-        self.__cb: t.Optional[t.Callable[[t.Any], t.Any]] = None
+        super().__init__(*args, **kwargs)
+        self._thread: Thread | None = None
 
     def start(self, cb: t.Callable[[t.Any], t.Any] | None = None) -> None:
-        self.__cb = cb
-        Thread.start(self)
+        self._thread = Thread(target=self._worker, args=(cb,), daemon=True)
+        self._thread.start()
 
-    def run(self) -> None:
+    def _worker(self, cb: t.Callable[[t.Any], t.Any] | None = None) -> None:
         # reset the loop in thread
         self.loop = None
-        self.ctrl.loop = self._ensure_ioloop()  # type: ignore[union-attr]
-        Arbiter.start(self, self.__cb)
+        self._ensure_ioloop()
+        self.ctrl.loop = self.loop  # type: ignore[union-attr]
+        super().start(cb)
 
     def stop(self) -> None:
-        self.loop.add_callback(Arbiter.stop, self)  # type: ignore[union-attr]
-        Thread.join(self)
+        if self.loop is not None:
+            self.loop.add_callback(super().stop)  # type: ignore[union-attr]
+        if self._thread is not None:
+            self._thread.join()
 
 
 def create_circus_socket_from_uri(
